@@ -1,3 +1,5 @@
+#![allow(clippy::manual_range_contains)]
+
 #[allow(non_camel_case_types)]
 pub mod ffi;
 
@@ -11,33 +13,35 @@ use crate::iter::*;
 
 use fallible_collections::FallibleVec;
 
+pub use rgb::Pod;
 pub use rgb::RGB;
 pub use rgb::RGBA8 as RGBA;
-pub use rgb::Pod;
 
-use std::os::raw::c_uint;
+use std::cmp;
+use std::convert::TryInto;
 use std::fmt;
+use std::fs;
 use std::mem;
+use std::os::raw::c_uint;
+use std::os::raw::c_void;
+use std::path::Path;
 use std::ptr;
 use std::slice;
-use std::cmp;
-use std::fs;
-use std::path::Path;
-use std::os::raw::c_void;
-use std::convert::TryInto;
 
-pub use crate::ffi::State;
 pub use crate::ffi::ColorType;
 pub use crate::ffi::CompressSettings;
-pub use crate::ffi::DecompressSettings;
-pub use crate::ffi::Time;
 pub use crate::ffi::DecoderSettings;
-pub use crate::ffi::FilterStrategy;
+pub use crate::ffi::DecompressSettings;
 pub use crate::ffi::EncoderSettings;
 pub use crate::ffi::ErrorCode;
+pub use crate::ffi::FilterStrategy;
+pub use crate::ffi::State;
+pub use crate::ffi::Time;
 
-pub use crate::ffi::Info;
 pub use crate::ffi::ColorMode;
+pub use crate::ffi::Info;
+
+pub use iter::ChunksIter;
 
 impl ColorMode {
     #[inline(always)]
@@ -50,6 +54,11 @@ impl ColorMode {
         self.colortype
     }
 
+    #[inline(always)]
+    pub fn set_colortype(&mut self, color: ColorType) {
+        self.colortype = color;
+    }
+
     #[inline(always)]
     pub fn bitdepth(&self) -> u32 {
         self.bitdepth
@@ -61,6 +70,18 @@ impl ColorMode {
         self.bitdepth = d;
     }
 
+    /// Set color depth to 8-bit palette and set the colors
+    pub fn set_palette(&mut self, palette: &[RGBA]) -> Result<(), Error> {
+        for &c in palette {
+            self.palette_add(c)?;
+        }
+        self.colortype = ColorType::PALETTE;
+        self.set_bitdepth(8);
+        self.palette_clear();
+        Ok(())
+    }
+
+    /// Reset to 0 colors
     #[inline]
     pub fn palette_clear(&mut self) {
         self.palette = None;
@@ -126,8 +147,8 @@ impl ColorMode {
     }
 
     /// is it a greyscale type? (only colortype 0 or 4)
-    #[cfg_attr(docsrs, doc(alias="is_grayscale_type"))]
-    #[cfg_attr(docsrs, doc(alias="is_gray"))]
+    #[cfg_attr(docsrs, doc(alias = "is_grayscale_type"))]
+    #[cfg_attr(docsrs, doc(alias = "is_gray"))]
     #[inline]
     pub fn is_greyscale_type(&self) -> bool {
         self.colortype == ColorType::GREY || self.colortype == ColorType::GREY_ALPHA
@@ -227,6 +248,13 @@ impl Time {
     }
 }
 
+impl Default for Info {
+    #[inline]
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Info {
     #[inline]
     pub fn new() -> Self {
@@ -245,21 +273,17 @@ impl Info {
 
     #[inline(always)]
     pub fn text_keys(&self) -> TextKeysIter<'_> {
-        TextKeysIter {
-            s: &self.texts,
-        }
+        TextKeysIter { s: &self.texts }
     }
 
-    #[deprecated(note="use text_keys")]
+    #[deprecated(note = "use text_keys")]
     pub fn text_keys_cstr(&self) -> TextKeysIter<'_> {
         self.text_keys()
     }
 
     #[inline(always)]
     pub fn itext_keys(&self) -> ITextKeysIter<'_> {
-        ITextKeysIter {
-            s: &self.itexts,
-        }
+        ITextKeysIter { s: &self.itexts }
     }
 
     /// use this to clear the texts again after you filled them in
@@ -291,12 +315,14 @@ impl Info {
         )
     }
 
+    /// Add literal PNG-data chunk unmodified to the unknown chunks
     #[inline]
     pub fn append_chunk(&mut self, position: ChunkPosition, chunk: ChunkRef<'_>) -> Result<(), Error> {
         self.unknown_chunks[position as usize].extend_from_slice(chunk.data);
         Ok(())
     }
 
+    /// Add custom chunk data to the file (if writing one)
     pub fn create_chunk<C: AsRef<[u8]>>(&mut self, position: ChunkPosition, chtype: C, data: &[u8]) -> Result<(), Error> {
         let chtype = chtype.as_ref();
         let chtype = match chtype.try_into() {
@@ -306,16 +332,24 @@ impl Info {
         rustimpl::add_chunk(&mut self.unknown_chunks[position as usize], &chtype, data)
     }
 
-    pub fn get<Name: AsRef<[u8]>>(&self, index: Name) -> Option<ChunkRef<'_>> {
+    /// Uses linear search to find a given chunk. You can use `b"PLTE"` syntax.
+    pub fn get<NameBytes: AsRef<[u8]>>(&self, index: NameBytes) -> Option<ChunkRef<'_>> {
         let index = index.as_ref();
-        self.unknown_chunks(ChunkPosition::IHDR)
-            .chain(self.unknown_chunks(ChunkPosition::PLTE))
-            .chain(self.unknown_chunks(ChunkPosition::IDAT))
+        self.try_unknown_chunks(ChunkPosition::IHDR)
+            .chain(self.try_unknown_chunks(ChunkPosition::PLTE))
+            .chain(self.try_unknown_chunks(ChunkPosition::IDAT))
+            .filter_map(|c| c.ok())
             .find(|c| c.is_type(index))
     }
 
+    #[deprecated(note = "use try_unknown_chunks")]
+    pub fn unknown_chunks(&self, position: ChunkPosition) -> ChunksIterFragile<'_> {
+        ChunksIterFragile::new(&self.unknown_chunks[position as usize])
+    }
+
+    /// Iterate over chunks that aren't part of image data. Only available if `remember_unknown_chunks` was set.
     #[inline]
-    pub fn unknown_chunks(&self, position: ChunkPosition) -> ChunksIter<'_> {
+    pub fn try_unknown_chunks(&self, position: ChunkPosition) -> ChunksIter<'_> {
         ChunksIter::new(&self.unknown_chunks[position as usize])
     }
 }
@@ -332,6 +366,12 @@ impl Encoder {
         Self::default()
     }
 
+    /// Switch color mode to 8-bit palette and set the colors
+    pub fn set_palette(&mut self, palette: &[RGBA]) -> Result<(), Error> {
+        self.state.info_raw_mut().set_palette(palette)?;
+        self.state.info_png_mut().color.set_palette(palette)
+    }
+
     /// If true, convert to output format
     #[inline(always)]
     pub fn set_auto_convert(&mut self, mode: bool) {
@@ -460,7 +500,7 @@ impl Decoder {
         self.state.remember_unknown_chunks(true_or_false);
     }
 
-    /// Decompress ICC profile from iCCP chunk
+    /// Decompress ICC profile from iCCP chunk. Only available if `remember_unknown_chunks` was set.
     #[inline(always)]
     pub fn get_icc(&self) -> Result<Vec<u8>, Error> {
         self.state.get_icc()
@@ -584,7 +624,7 @@ impl State {
         self.decoder.remember_unknown_chunks = true_or_false;
     }
 
-    /// Decompress ICC profile from iCCP chunk
+    /// Decompress ICC profile from iCCP chunk. Only available if `remember_unknown_chunks` was set.
     pub fn get_icc(&self) -> Result<Vec<u8>, Error> {
         let iccp = self.info_png().get("iCCP");
         if iccp.is_none() {
@@ -645,7 +685,7 @@ impl State {
     #[deprecated(note = "Use Encoder type instead of State")]
     pub fn encode<PixelType: rgb::Pod>(&mut self, image: &[PixelType], w: usize, h: usize) -> Result<Vec<u8>, Error> {
         let image = buffer_for_type(image, w, h, self.info_raw.colortype, self.info_raw.bitdepth)?;
-        Ok(rustimpl::lodepng_encode(image, w as c_uint, h as c_uint, self)?)
+        rustimpl::lodepng_encode(image, w as c_uint, h as c_uint, self)
     }
 
     #[deprecated(note = "Use Encoder type instead of State")]
@@ -670,9 +710,9 @@ impl Default for State {
     }
 }
 
-#[cfg_attr(docsrs, doc(alias="Gray"))]
+#[cfg_attr(docsrs, doc(alias = "Gray"))]
 pub use rgb::alt::Gray as Grey;
-#[cfg_attr(docsrs, doc(alias="GrayAlpha"))]
+#[cfg_attr(docsrs, doc(alias = "GrayAlpha"))]
 pub use rgb::alt::GrayAlpha as GreyAlpha;
 
 /// Bitmap types.
@@ -683,17 +723,17 @@ pub use rgb::alt::GrayAlpha as GreyAlpha;
 pub enum Image {
     /// Bytes of the image. See bpp how many pixels per element there are
     RawData(Bitmap<u8>),
-    #[cfg_attr(docsrs, doc(alias="Gray"))]
+    #[cfg_attr(docsrs, doc(alias = "Gray"))]
     Grey(Bitmap<Grey<u8>>),
-    #[cfg_attr(docsrs, doc(alias="Gray16"))]
+    #[cfg_attr(docsrs, doc(alias = "Gray16"))]
     Grey16(Bitmap<Grey<u16>>),
-    #[cfg_attr(docsrs, doc(alias="GrayAlpha"))]
+    #[cfg_attr(docsrs, doc(alias = "GrayAlpha"))]
     GreyAlpha(Bitmap<GreyAlpha<u8>>),
-    #[cfg_attr(docsrs, doc(alias="GrayAlpha16"))]
+    #[cfg_attr(docsrs, doc(alias = "GrayAlpha16"))]
     GreyAlpha16(Bitmap<GreyAlpha<u16>>),
-    #[cfg_attr(docsrs, doc(alias="RGBA8"))]
+    #[cfg_attr(docsrs, doc(alias = "RGBA8"))]
     RGBA(Bitmap<RGBA>),
-    #[cfg_attr(docsrs, doc(alias="RGB8"))]
+    #[cfg_attr(docsrs, doc(alias = "RGB8"))]
     RGB(Bitmap<RGB<u8>>),
     RGBA16(Bitmap<rgb::RGBA<u16>>),
     RGB16(Bitmap<RGB<u16>>),
@@ -755,11 +795,7 @@ impl<PixelType: rgb::Pod> Bitmap<PixelType> {
             }
             out
         };
-        Ok(Self {
-            buffer,
-            width,
-            height,
-        })
+        Ok(Self { buffer, width, height })
     }
 }
 
@@ -910,7 +946,7 @@ fn buffer_for_type<PixelType: rgb::Pod>(image: &[PixelType], w: usize, h: usize,
 #[inline]
 pub fn encode_memory<PixelType: rgb::Pod>(image: &[PixelType], w: usize, h: usize, colortype: ColorType, bitdepth: c_uint) -> Result<Vec<u8>, Error> {
     let image = buffer_for_type(image, w, h, colortype, bitdepth)?;
-    Ok(rustimpl::lodepng_encode_memory(image, w as u32, h as u32, colortype, bitdepth)?)
+    rustimpl::lodepng_encode_memory(image, w as u32, h as u32, colortype, bitdepth)
 }
 
 /// Same as `encode_memory`, but always encodes from 32-bit RGBA raw image
@@ -983,6 +1019,7 @@ impl<'a> ChunkRef<'a> {
     }
 
     #[inline(always)]
+    #[allow(clippy::len_without_is_empty)]
     pub fn len(&self) -> usize {
         lodepng_chunk_length(self.data)
     }
@@ -1157,8 +1194,8 @@ impl Default for EncoderSettings {
 
 #[cfg(test)]
 mod test {
-    use std::mem;
     use super::*;
+    use std::mem;
 
     #[test]
     fn pixel_sizes() {
@@ -1179,8 +1216,8 @@ mod test {
     fn create_and_destroy2() {
         Encoder::new().info_png();
         Encoder::new().info_png_mut();
-        Encoder::new().clone().info_raw();
-        Encoder::new().clone().info_raw_mut();
+        Encoder::new().info_raw();
+        Encoder::new().info_raw_mut();
     }
 
     #[test]
@@ -1200,21 +1237,21 @@ mod test {
         let mut state = Encoder::new();
         {
             let info = state.info_png_mut();
-            for _ in info.unknown_chunks(ChunkPosition::IHDR) {
+            for _ in info.try_unknown_chunks(ChunkPosition::IHDR) {
                 panic!("no chunks yet");
             }
 
-            let testdata = &[1,2,3];
-            info.create_chunk(ChunkPosition::PLTE, &[255,0,100,32], testdata).unwrap();
-            assert_eq!(1, info.unknown_chunks(ChunkPosition::PLTE).count());
+            let testdata = &[1, 2, 3];
+            info.create_chunk(ChunkPosition::PLTE, &[255, 0, 100, 32], testdata).unwrap();
+            assert_eq!(1, info.try_unknown_chunks(ChunkPosition::PLTE).count());
 
             info.create_chunk(ChunkPosition::IHDR, "foob", testdata).unwrap();
-            assert_eq!(1, info.unknown_chunks(ChunkPosition::IHDR).count());
+            assert_eq!(1, info.try_unknown_chunks(ChunkPosition::IHDR).count());
             info.create_chunk(ChunkPosition::IHDR, "foob", testdata).unwrap();
-            assert_eq!(2, info.unknown_chunks(ChunkPosition::IHDR).count());
+            assert_eq!(2, info.try_unknown_chunks(ChunkPosition::IHDR).count());
 
-            for _ in info.unknown_chunks(ChunkPosition::IDAT) {}
-            let chunk = info.unknown_chunks(ChunkPosition::IHDR).next().unwrap();
+            for _ in info.try_unknown_chunks(ChunkPosition::IDAT) {}
+            let chunk = info.try_unknown_chunks(ChunkPosition::IHDR).next().unwrap().unwrap();
             assert_eq!("foob".as_bytes(), chunk.name());
             assert!(chunk.is_type("foob"));
             assert!(!chunk.is_type("foobar"));
@@ -1229,7 +1266,7 @@ mod test {
         let mut dec = Decoder::new();
         dec.remember_unknown_chunks(true);
         dec.decode(img).unwrap();
-        let chunk = dec.info_png().unknown_chunks(ChunkPosition::IHDR).next().unwrap();
+        let chunk = dec.info_png().try_unknown_chunks(ChunkPosition::IHDR).next().unwrap().unwrap();
         assert_eq!("foob".as_bytes(), chunk.name());
         dec.info_png().get("foob").unwrap();
     }
